<?php

namespace Insallah;

class Tournament
{
    const IS_ENABLED = true;
    const SEND_CHAT_MESSAGE = 3;
    const START_TOURNAMENT = 6;
    const PRIZE1 = 25;
    const PRIZE2 = 10;
    const PRIZE3 = 5;
    const MIN_EXP = 175000;
    const CHAT_MESSAGE = 'Внимание! Турнир для %d уровня открыт! Поспешите принять участие в Зале Воинов!';
    const VICTORY_MESSAGE = 'Турнир для %d уровней завершён! Первое место: %s. Второе место: %s. Третье место: %s.';
    const DELAY1 = '+12hour';
    const DELAY2 = '+6hour';
    const DELAY3 = '+3hour';

    /** Формула рассчёта разрешённой стоимости одетых предметов.
     *
     * @param int $int
     *
     * @return float|int
     */
    public static function ekrOverpriceFormula($int)
    {
        return ($int - 7) * 151;
    }

    /**
     * Проверяем может ли игрок учавствовать в турнире и либо присоединяем его, либо обламываем.
     *
     * @param int $user_id
     *
     * @return void
     */
    public function join($user_id)
    {
        /** Возвращает level, или 0 если провалены проверки. */
        $userLevel = TournamentModel::getUserLevel($user_id);
        if (
            $userLevel === 0 ||
            TournamentModel::isEkrOverpriced($user_id, $userLevel) ||
            !TournamentModel::isEnoughExperience($user_id) ||
            TournamentModel::IsRestrictedToJoin($user_id) ||
            TournamentModel::isStarted($user_id)
        ) {
            return;
        }

        $mq = TournamentModel::getWaitingMembersQuantity($userLevel);
        if (empty($mq)) {
            TournamentModel::createTournament($userLevel);
        }

        TournamentModel::joinTournament($user_id, $userLevel);
        $mq++;

        if ($mq == self::SEND_CHAT_MESSAGE) {
            TournamentModel::sysMessage(sprintf(self::CHAT_MESSAGE, $userLevel));
        }

        if ($mq == self::START_TOURNAMENT) {
            TournamentModel::startTournament($userLevel);
        }
    }

    /**
     * Проверка для крона\конфига. Выбивает проигравших и перезапускает поединки во всех турнирах.
     * @return void
     */
    public function startAllBattles()
    {
        $db = new Db();
        $db::sql('delete from tournaments where start_time + date_add(start_time,interval 30 minute) < unix_timestamp()');
        TournamentModel::removeFighter(TournamentModel::getLooser());
        $tournament_levels = $db::getColumn('select tid from tournaments where start_time = -1');
        foreach ($tournament_levels as $level) {
            $alive_fighters = TournamentModel::getFreeFighters($level);
            if (count($alive_fighters) > 1) {
                $this->start_battle($alive_fighters);
            } elseif (count($alive_fighters) === 1) {
                TournamentModel::removeFighter($alive_fighters[0]);
                $this->victory($level);
            }
        }
    }

    /**
     * Получаем пары бойцов и начинаем их поединки.
     *
     * @param array $fighters_list
     *
     * @return void
     */
    private function start_battle(array $fighters_list)
    {
        $teams = TournamentModel::getFightersTeams($fighters_list);
        foreach ($teams as $team) {
            if (count($team) === 2) {
                TournamentModel::startBattle($team[0], $team[1]);
            }
        }
    }

    /**
     * Награждаем победителей турнира, чистим базу.
     *
     * @param int $tournament_id
     *
     * @return void
     */
    private function victory($tournament_id)
    {
        $winners = TournamentModel::getWinners($tournament_id);
        sleep(1);
        TournamentModel::givePrizeItems($winners[1], self::PRIZE1);
        TournamentModel::givePrizeItems($winners[2], self::PRIZE2);
        TournamentModel::givePrizeItems($winners[3], self::PRIZE3);
        sleep(1);
        TournamentModel::giveDelay($winners[1], strtotime(self::DELAY1));
        TournamentModel::giveDelay($winners[2], strtotime(self::DELAY2));
        TournamentModel::giveDelay($winners[3], strtotime(self::DELAY3));
        sleep(1);
        TournamentModel::sysMessage(sprintf(self::VICTORY_MESSAGE,
            $tournament_id,
            TournamentModel::uidToLogin($winners[1]),
            TournamentModel::uidToLogin($winners[2]),
            TournamentModel::uidToLogin($winners[3])));
        sleep(3);
        TournamentModel::destroyTournament($tournament_id);
    }
}