<?php

namespace Insallah;

class TournamentModel
{

    /**
     * проверка уровня, стоимости эквипа, прочие проверки, что персонаж свободен
     * таймер ожидания 30 минут
     *
     * @param int $uid
     *
     * @return int
     */
    public static function getUserLevel($uid)
    {
        $db = new Db();
        $level = $db::getValue('select level from users where id = ? and level between 8 and 12 and battle = 0', [$uid]);
        return $level ? $level : 0;
    }

    /**
     * @param int $uid
     * @param int $level
     *
     * @return bool
     */
    public static function isEkrOverpriced($uid, $level = null)
    {
        $db = new Db();
        if (is_null($level)) {
            $level = $db::getValue('select level from users where id = ?', [$uid]);
        }
        $wearedItemsEkrPrice = $db::getValue('select sum(2price) from items_users where inOdet > 0 and uid = ?', [$uid]);
        return $wearedItemsEkrPrice > Tournament::ekrOverpriceFormula($level);
    }

    /**
     * @param int $uid
     *
     * @return bool
     */
    public static function isEnoughExperience($uid)
    {
        $db = new Db();
        return $db::getValue('select exp from stats where id = ?', [$uid]) >= Tournament::MIN_EXP;
        }

    /**
     * @param int $uid
     *
     * @return bool
     */
    public static function IsRestrictedToJoin($uid)
    {
        $db = new Db();
        $delayEffect = $db::getValue('select count(*) from eff_users where uid = ? and id_eff = 486 and `delete` = 0', [$uid]);
        return (bool)$delayEffect;
    }

    /**
     * @param $tid
     *
     * @return bool
     */
    public static function isStarted($tid)
    {
        $db = new Db();
        $status = $db::getValue('select count(*) from tournaments where start_time = -1 and tid = ?', [$tid]);
        return (bool)$status;
    }

    /**
     * Считаем сколько игроков ждут в заявке на турнир.
     *
     * @param int $tid
     *
     * @return int
     */
    public static function getWaitingMembersQuantity($tid)
    {
        $db = new Db();
        return $db::getValue('select count(*) from tournaments_users where tid = ?', [$tid]);
    }

    /**
     * Создание нового турнира.
     *
     * @param int $tid
     *
     * @return void
     */
    public static function createTournament($tid)
    {
        $db = new Db();
        $db::sql('insert into tournaments (tid) values (?)', [$tid]);
    }

    /**
     * Игрок присоединяется к турниру и телепортируется в турнирную комнату.
     *
     * @param int $uid
     * @param int $tid
     *
     * @return void
     */
    public static function joinTournament($uid, $tid)
    {
        /** Кастомные комнаты 25008 - 25012. */
        $room_id = 25000 + $tid;
        $db = new Db();
        $db::sql('insert into tournaments_users (tid, uid) values (?, ?)', [$tid, $uid]);
        self::teleport($uid, $room_id);
    }

    /**
     * Старт турнира.
     *
     * @param int $tid
     *
     * @return void
     */
    public static function startTournament($tid)
    {
        $db = new Db();
        $db::sql('update tournaments set start_time = -1 where tid = ?', [$tid]);
    }

    /**
     * Чистим базы от прошедшего турнира.
     *
     * @param int $tid
     *
     * @return void
     */
    public static function destroyTournament($tid)
    {
        $db = new Db();
        //Убедиться что в базе настроен foreign_keys и последует автоочистка tournaments_users !!!
        $db::sql('delete from tournaments where tid = ?', [$tid]);
    }

    /**
     * Получаем список бойцов и бьём их на пары. Возвращаем списки пар + 1 последний без пары если есть.
     *
     * @param array $fighters_list
     *
     * @return array
     */
    public static function getFightersTeams(array $fighters_list)
    {
        $db = new Db();
        $query = sprintf("select id from users where battle = 0 and id in (%s)", implode(', ', $fighters_list));
        return array_chunk($db::getColumn($query), 2);
    }

    /**
     * Выбираем живых бойцов не сражающихся в данный момент.
     *
     * @param int $tid
     *
     * @return array
     */
    public static function getFreeFighters($tid)
    {
        $db = new Db();
        return $db::getColumn('select uid from tournaments_users where tid = ? and death_time = 0 order by rand()', [$tid]);
    }

    /**
     * Выбираем победителей. Смещаем массив, чтобы возврат шёл с единицы.
     *
     * @param int $tid
     *
     * @return array
     */
    public static function getWinners($tid)
    {
        $db = new Db();
        $winners = $db::getColumn('select uid from tournaments_users where tid = ? order by death_time desc limit 3', [$tid]);
        return [
            1 => $winners[0],
            2 => $winners[1],
            3 => $winners[2]
        ];
    }

    /**
     * Пробуем выкусить проигравшего в последней турнирной битве и удалить эту самую битву во избежание.
     * @return mixed
     */
    public static function getLooser()
    {
        $query = '
            select uid, battle 
            from 
                battle_users, 
                (select id, team_win 
                 from battle 
                 where 
                     team_win > 0 and 
                     typeBattle = 25000 
                 order by time_over desc 
                 limit 1) as last_battle 
            where 
                battle_users.battle = last_battle.id and 
                battle_users.team != last_battle.team_win and
                battle_users.uid in (select uid from tournaments_users where death_time = 0)';
        $db = new Db;
        $row = $db::getRow($query);
        return $row['uid'];
    }

    /**
     * Выбывший из турнира покидает комнату и получает время смерти.
     *
     * @param int $uid
     *
     * @return void
     */
    public static function removeFighter($uid)
    {
        if (!$uid) return;
        $db = new Db();
        $db::sql('update tournaments_users set death_time = unix_timestamp() where death_time = 0 and uid = ?', [$uid]);
        self::teleport($uid, 9);
        //fixme: Классы не подключаются друг к другу. Нужно менять архитектуру игры. :(
        Db::sql("update users_achiv set trn = trn + 1 where id = ?", [$uid]);
        //(new Achievements(\user::start()))->updateCounter('trn');
    }

    /**
     * Узнаём id турнира по id игрока.
     *
     * @param int $uid
     *
     * @return mixed
     */
    public static function getTournamentIdByUserId($uid)
    {
        $db = new Db();
        return $db::getValue('select tid from tournaments_users where uid = ?', [$uid]);
    }

    /**
     * Попробуем стартонуть поединок.
     * 25000 - Уникальный id поединка под турниры.
     * noinc - запрет на вмешательство
     * invis - невидимый бой
     *
     * @param int $uid1
     * @param int $uid2
     *
     * @return void
     */
    public static function startBattle($uid1, $uid2)
    {
        $db = new Db();
        $db::exec('insert into battle (city, time_start, timeout, type, invis, noinc, travmChance, typeBattle) 
        values (\'capitalcity\', unix_timestamp(), 60, 0, 1, 1, 0, 25000)');
        $db::sql('update stats set team = 1, hpNow = hpAll, mpNow = mpAll where id = ?', [$uid1]);
        $db::sql('update stats set team = 2, hpNow = hpAll, mpNow = mpAll where id = ?', [$uid2]);
        $db::sql('update users set battle = ? where id in (?, ?)', [$db::lastInsertId(), $uid1, $uid2]);
    }

    /**
     * Узнаём логин персонажа по его id.
     *
     * @param int $uid
     *
     * @return mixed
     */
    public static function uidToLogin($uid)
    {
        $db = new Db();
        return $db::getValue('select login from users where id = ?', [$uid]);
    }

    /**
     * Телепорт по комнатам.
     *
     * @param int $uid
     * @param int $room_id
     *
     * @return void
     */
    private static function teleport($uid, $room_id)
    {
        $db = new Db();
        $db::sql('update users set room = ? where id = ?', [$room_id, $uid]);
    }

    /**
     * Нет проверок $message потому что оно всегда задаётся в коде и игрок на него не влияет.
     *
     * @param string $message
     *
     * @return void
     */
    public static function sysMessage($message)
    {
        if (!empty($message)) {
        $db = new Db();
            $message = "<span style='font-weight: bold; color: forestgreen;'>$message</span>";
            $db::sql('insert into chat (time, type, text, new, da) values (unix_timestamp(), 6, ?, 1, 1)', [$message]);
        }
    }

    /**
     * Генерирует множественный запрос сразу на $quantity однотипных предметов в инвентарь пользователя $uid.
     *
     * @param int $uid
     * @param int $quantity
     *
     * @return void
     */
    public static function givePrizeItems($uid, $quantity)
    {
        $query = 'insert into items_users (item_id, uid, data, iznosMAX, lastUPD, time_create) 
                    values (4754, :uid, :data, 1, unix_timestamp(), unix_timestamp())';
        $args = [
            'uid' => $uid,
            'data' => 'nosale=1|musor=1|sudba=' . self::uidToLogin($uid) . '|lvl=8|tr_s1=0|tr_s2=0|tr_s3=0|tr_s4=0'
        ];
        $db = new Db();
        $stmt = $db::prepare($query);
        for ($i = 0; $i < $quantity; $i++) {
            $stmt->execute($args);
        }
    }

    /** Эффект-ограничитель на участие в турнире.
     * @param $uid
     * @param $unix_time
     *
     * @return void
     */
    public static function giveDelay($uid, $unix_time)
    {
        $db = new Db();
        $query = 'insert into eff_users (id_eff, uid, name, timeUse) VALUES (?,?,?,?)';
        $args = [486, $uid, 'Призёр городского турнира!', $unix_time];
        $db::sql($query, $args);
    }
}